<?php

namespace Bixo\Offplan\Http\Controllers;

use Exception;
use Bixo\Offplan\Actions\BookingWorkflow;
use Bixo\Offplan\Http\Requests\BookingWorkflowRequest;
use Bixo\Offplan\Http\Resources\BookingResource;
use Bixo\Offplan\Models\Booking;
use Litepie\Http\Controllers\ActionController as BaseController;

/**
 * Workflow controller class.
 *
 */
class BookingWorkflowController extends BaseController
{
    /**
     * Action controller function for booking.
     *
     * @param Model $booking
     * @param action next action for the booking.
     *
     * @return Response
     */
    public function __invoke(BookingWorkflowRequest $request, Booking $booking, $trasition)
    {
        try {
            $request = $request->all();
            $booking = BookingWorkflow::run($trasition, $booking, $request);
            $data = new BookingResource($booking);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('offplan::booking.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('offplan/booking/' . $booking->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('offplan/booking/' . $booking->getRouteKey()))
                ->redirect();
        }
    }


    /**
     * Get the booking and user for the workflow.
     *
     * @param PublicRequest $request
     * @param string $booking
     * @param string $user
     *
     * @return Response
     */
    public function get(PublicRequest $request, $booking, $user)
    {
        if (!$request->hasValidSignature()) {
            abort(403, 'This url is expired!');
        }

        $auth = Auth::guard()->getProvider()->getModel();
        $user = $auth::findBySignedId($user);
        Auth::guard()->login($user);
        $booking = Booking::findBySignedId($booking);
        $transitions = $this->transitions($booking, $user);
        $user = user();
        Auth::logout();
        return $this->response->setMetaTitle(trans('offplan::booking.name') . ' ' . trans('Approvals'))
            ->view('offplan::public.booking.workflow')
            ->layout('mobile')
            ->data(compact('booking', 'user', 'transitions'))
            ->output();
    }

    /**
     * Display the workflow form for the booking.
     *
     * @param PublicRequest $request
     * @param string $booking
     * @param string $user
     *
     * @return Response
     */
    public function post(PublicRequest $request, $booking, $user)
    {
        if (!$request->hasValidSignature()) {
            abort(403, 'This url is expired!');
        }
        $transition = $request->transition;
        $auth = Auth::guard()->getProvider()->getModel();
        $user = $auth::findBySignedId($user);
        Auth::guard()->login($user);
        $booking = Booking::findBySignedId($booking);
        $user = user();
        $booking = BookingWorkflow::run($transition, $booking, $request->all());
        Auth::logout();

        return response()->json(
            [
                'status' => 'success',
                'url' => $request->url(),
            ]
        );
    }

    private function transitions($booking, $user)
    {
        $transitions = [];
        foreach ($booking->workflow()->transitions($booking) as $key => $value) {
            $name = $value->getName();
            $array['url'] = URL::temporarySignedRoute('litecms.booking.workflow', now()->addMinutes(3), [
                'guard' => 'admin',
                'transition' => $name,
                'booking' => $booking->getSignedId(now()->addMinutes(3)),
                'user' => $user->getSignedId(now()->addMinutes(3)),
                'trans' => 'en',
            ]
            );
            $array['name'] = $name;
            $array['key'] = $name;
            $array['form'] = $booking->workflow()->form($value);
            $array['label'] = trans('offplan::booking.workflow.' . $name);
            $transitions[] = $array;
        }
        return $transitions;
    }
}
